package com.ozs.web.controller.pm;

import java.util.List;

import com.github.pagehelper.PageInfo;
import com.ozs.common.core.domain.model.LoginUser;
import com.ozs.framework.web.service.TokenService;
import com.ozs.pm.doman.vo.requestVo.PmBookBuildingReqVo;
import com.ozs.pm.doman.vo.requestVo.PmReqVo;
import com.ozs.pm.doman.vo.responseVo.PmDemandResVo;
import com.ozs.pm.service.IPmDemandService;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ozs.common.core.controller.BaseController;
import com.ozs.common.core.domain.AjaxResult;

import javax.servlet.http.HttpServletRequest;

/**
 * 采购需求Controller
 *
 * @author ruoyi
 * @date 2023-01-16
 */
@RestController
@RequestMapping("/system/demand")
public class PmDemandController extends BaseController {
    @Autowired
    private IPmDemandService pmDemandService;
    @Autowired
    private TokenService tokenService;

    /**
     * 查询采购需求列表
     */
    @ApiOperation(value = "查询采购需求列表", notes = "参数非必传")
    @PostMapping("/list")
    public AjaxResult list(@RequestBody PmReqVo pmReqVo) {
        List<PmDemandResVo> list = pmDemandService.selectPmDemandList(pmReqVo);
        PageInfo<PmDemandResVo> page = new PageInfo<>(list);
        return success(page);
    }

    /**
     * 获取采购需求详细信息
     */
    @ApiOperation(value = "获取采购需求详细信息", notes = "必传demandId,其他字段不传")
    @PostMapping("/view")
    public AjaxResult view(@RequestBody PmReqVo pmReqVo) {
        return success(pmDemandService.selectPmDemandByDemandId(pmReqVo.getDemandId()));
    }

    /**
     * 需求建档
     */
    @ApiOperation(value = "需求建档", notes = "必传demandId,根据类型必传pmDemandEngineeringResponseVo、pmDemandEquipResponseVo、pmDemandMaterialsResponseVo、pmDemandServeResponseVo")
    @PostMapping("/bookBuilding")
    public AjaxResult bookBuilding(@RequestBody PmBookBuildingReqVo pmBookBuildingReqVo) {
        return toAjax(pmDemandService.bookBuilding(pmBookBuildingReqVo));
    }

    /**
     * 查看流程图
     */
    @ApiOperation(value = "查看流程图", notes = "必传demandId,其他字段不传")
    @PostMapping("/viewFlowChart")
    public AjaxResult viewFlowChart(@RequestBody PmReqVo pmRequestVo) {
        return success(pmDemandService.viewFlowChart(pmRequestVo.getDemandId()));
    }


    @ApiOperation(value = "提交采购需求", notes = "必传demandId,其他字段不传")
    @PostMapping("/commit")
    public AjaxResult commit(@RequestBody PmReqVo pmReqVo, HttpServletRequest request) {
        LoginUser loginUser = tokenService.getLoginUser(request);
        pmReqVo.setUpdateBy(String.valueOf(loginUser.getUserId()));
        return toAjax(pmDemandService.commit(pmReqVo));
    }

    @ApiOperation(value = "审核采购需求通过", notes = "必传demandId和上传附件，其他字段不传")
    @PostMapping("/reviewTo")
    public AjaxResult reviewTo(@RequestBody PmReqVo pmReqVo, HttpServletRequest request) {
        LoginUser loginUser = tokenService.getLoginUser(request);
        pmReqVo.setUpdateBy(String.valueOf(loginUser.getUserId()));
        return toAjax(pmDemandService.reviewTo(pmReqVo));
    }

    @ApiOperation(value = "审核采购需求退回", notes = "必传demandId和退回原因，其他字段不传")
    @PostMapping("/reviewReturn")
    public AjaxResult reviewReturn(@RequestBody PmReqVo pmReqVo, HttpServletRequest request) {
        LoginUser loginUser = tokenService.getLoginUser(request);
        pmReqVo.setUpdateBy(String.valueOf(loginUser.getUserId()));
        return toAjax(pmDemandService.reviewReturn(pmReqVo));
    }

}
