package com.ozs.pm.service.impl;

import java.util.*;

import com.ozs.common.enums.*;
import com.ozs.common.utils.DateUtils;
import com.ozs.common.utils.StringUtils;
import com.ozs.common.utils.bean.BeanUtils;
import com.ozs.pm.doman.*;
import com.ozs.pm.doman.vo.requestVo.PmBookBuildingReqVo;
import com.ozs.pm.doman.vo.requestVo.PmReqVo;
import com.ozs.pm.doman.vo.responseVo.*;
import com.ozs.pm.mapper.PmDemandMapper;
import com.ozs.pm.service.*;
import com.ozs.system.domain.SysFileInfo;
import com.ozs.system.domain.SysFileRef;
import com.ozs.system.domain.vo.responseVo.SysDeptResponseVo;
import com.ozs.system.service.ISysDeptService;
import com.ozs.system.service.ISysDictDataService;
import com.ozs.system.service.SysFileRefService;
import com.ozs.system.service.SysFileService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;


/**
 * 采购需求Service业务层处理
 *
 * @author ruoyi
 * @date 2023-01-16
 */
@Service
public class PmDemandServiceImpl implements IPmDemandService
{
    @Autowired
    private PmDemandMapper pmDemandMapper;
    @Autowired
    private IPmDemandServeService iPmDemandServeService;
    @Autowired
    private IPmDemandMaterialsService iPmDemandMaterialsService;
    @Autowired
    private IPmDemandEquipService iPmDemandEquipService;
    @Autowired
    private IPmDemandEngineeringService iPmDemandEngineeringService;
    @Autowired
    private ISysDeptService deptService;

    @Autowired
    private ISysDictDataService dictDataService;
    @Autowired
    private SysFileService fileService;
    @Autowired
    private SysFileRefService sysFileRefService;

    /**
     * 查询采购需求
     *
     * @param demandId 采购需求主键
     * @return 采购需求
     */
    @Override
    public PmDemandResVo selectPmDemandByDemandId(Long demandId) {
        PmDemand pmDemand = pmDemandMapper.selectPmDemandByDemandId(demandId);
        if(pmDemand != null){
            PmDemandResVo vo = new PmDemandResVo();
            BeanUtils.copyProperties(pmDemand, vo);

            SysDeptResponseVo sysDeptResponseVo = (SysDeptResponseVo) deptService.selectDeptById(vo.getPurchaseDeptId()).get("sysDept");
            if(sysDeptResponseVo != null){
                vo.setPurchaseDeptName(sysDeptResponseVo.getDeptName());
            }

            String purchaseServicesName = dictDataService.selectDictLabel("purchase_services", vo.getPurchaseServices());
            if(StringUtils.isNotEmpty(purchaseServicesName)){
                vo.setPurchaseServicesName(purchaseServicesName);
            }

            HashMap<String, String> demandFileMap = getFileMap(vo.getDemandId(),SysFileRefEnum.PM_DEMAND.getType());
            if(demandFileMap != null){
                vo.setDemandFileMap(demandFileMap);
            }

            HashMap<String, String> auditFileMap = getFileMap(vo.getDemandId(),SysFileRefEnum.PM_DEMAND_EXAMINE.getType());
            if(auditFileMap != null){
                vo.setAuditFileMap(auditFileMap);
            }

            //项目类型
            for (ProjectTypes value : ProjectTypes.values()) {
                if (vo.getProjectType() != null && vo.getProjectType().equals(value.getCode())) {
                    vo.setProjectTypeName(value.getInfo());
                    break;
                }
            }
            //是否为超限额计划
            for (IsExcess value : IsExcess.values()) {
                if (vo.getIsExcess() != null && vo.getIsExcess().equals(value.getCode())) {
                    vo.setIsExcessName(value.getInfo());
                    break;
                }
            }

            //采购方式
            for (PlanPurchaseMode value : PlanPurchaseMode.values()) {
                if (vo.getPurchaseMode() != null && vo.getPurchaseMode().equals(value.getCode())) {
                    vo.setPurchaseModeName(value.getInfo());
                    break;
                }
            }
            //项目属性
            for (ProjectAttribute value : ProjectAttribute.values()) {
                if (vo.getProjectAttr() != null && vo.getProjectAttr().equals(value.getCode())) {
                    vo.setProjectAttrName(value.getInfo());
                    break;
                }
            }
            //预警状态
            for (WarnStatus value : WarnStatus.values()) {
                if (vo.getWarnStatus() != null && vo.getWarnStatus().equals(value.getCode())) {
                    vo.setWarnStatusName(value.getInfo());
                    break;
                }
            }

            //项目状态
            for (PmProjectStatus value : PmProjectStatus.values()) {
                if (vo.getProjectStatus() != null && vo.getProjectStatus().equals(value.getCode())) {
                    vo.setProjectStatusName(value.getInfo());
                    break;
                }
            }

            if(ProjectTypes.EQUIPMENTTYPE.getCode().equals(vo.getProjectType())){//装备类
                PmDemandEquip obj = new PmDemandEquip();
                obj.setDemandId(demandId);
                List<PmDemandEquip> list = iPmDemandEquipService.selectPmDemandEquipList(obj);
                if(list !=null && list.size() > 0){
                    PmDemandEquipResVo responseVo = new PmDemandEquipResVo();
                    BeanUtils.copyProperties(list.get(0),responseVo);
                    //采购方式建议
                    for (ProcurementMethodSuggest value : ProcurementMethodSuggest.values()) {
                        if (responseVo.getProcurementMethodSuggest() != null && responseVo.getProcurementMethodSuggest().equals(value.getCode())) {
                            responseVo.setProcurementMethodSuggestName(value.getInfo());
                            break;
                        }
                    }

                    HashMap<String, String> fileMap1 = getFileMap(vo.getDemandId(),SysFileRefEnum.PM_DEMAND_EQUIP.getType());
                    if(fileMap1 != null){
                        responseVo.setFileMap(fileMap1);
                    }

                    vo.setPmDemandEquipResponseVo(responseVo);
                }
            } else if(ProjectTypes.MATERIALTYPE.getCode().equals(vo.getProjectType())){ //物资类
                PmDemandMaterials obj = new PmDemandMaterials();
                obj.setDemandId(demandId);
                List<PmDemandMaterials> list = iPmDemandMaterialsService.selectPmDemandMaterialsList(obj);
                if(list !=null && list.size() > 0){
                    PmDemandMaterialsResVo responseVo = new PmDemandMaterialsResVo();
                    BeanUtils.copyProperties(list.get(0),responseVo);
                    //采购方式建议
                    for (ProcurementMethodSuggest value : ProcurementMethodSuggest.values()) {
                        if (responseVo.getProcurementMethodSuggest() != null && responseVo.getProcurementMethodSuggest().equals(value.getCode())) {
                            responseVo.setProcurementMethodSuggestName(value.getInfo());
                            break;
                        }
                    }
                    HashMap<String, String> fileMap2 = getFileMap(vo.getDemandId(),SysFileRefEnum.PM_DEMAND_MATERIALS.getType());
                    if(fileMap2 != null){
                        responseVo.setFileMap(fileMap2);
                    }
                    vo.setPmDemandMaterialsResponseVo(responseVo);
                }
            } else if(ProjectTypes.SERVICESTYPE.getCode().equals(vo.getProjectType())){//服务类
                PmDemandServe obj = new PmDemandServe();
                obj.setDemandId(demandId);
                List<PmDemandServe> list = iPmDemandServeService.selectPmDemandServeList(obj);
                if(list !=null && list.size() > 0){
                    PmDemandServeResVo responseVo = new PmDemandServeResVo();
                    BeanUtils.copyProperties(list.get(0),responseVo);
                    //采购方式建议
                    for (ProcurementMethodSuggest value : ProcurementMethodSuggest.values()) {
                        if (responseVo.getProcurementMethodSuggest() != null && responseVo.getProcurementMethodSuggest().equals(value.getCode())) {
                            responseVo.setProcurementMethodSuggestName(value.getInfo());
                            break;
                        }
                    }
                    HashMap<String, String> fileMap3 = getFileMap(vo.getDemandId(),SysFileRefEnum.PM_DEMAND_SERVE.getType());
                    if(fileMap3 != null){
                        responseVo.setFileMap(fileMap3);
                    }
                    vo.setPmDemandServeResponseVo(responseVo);
                }
            } else if(ProjectTypes.PLANTOEXAMINETYPE.getCode().equals(vo.getProjectType())){//工程类
                PmDemandEngineering obj = new PmDemandEngineering();
                obj.setDemandId(demandId);
                List<PmDemandEngineering> list = iPmDemandEngineeringService.selectPmDemandEngineeringList(obj);
                if(list !=null && list.size() > 0){
                    PmDemandEngineeringResVo responseVo = new PmDemandEngineeringResVo();
                    BeanUtils.copyProperties(list.get(0),responseVo);
                    //采购方式建议
                    for (ProcurementMethodSuggest value : ProcurementMethodSuggest.values()) {
                        if (responseVo.getProcurementMethodSuggest() != null && responseVo.getProcurementMethodSuggest().equals(value.getCode())) {
                            responseVo.setProcurementMethodSuggestName(value.getInfo());
                            break;
                        }
                    }
                    //项目类别
                    for (EngineeringProjectCategory value : EngineeringProjectCategory.values()) {
                        if (responseVo.getProjectCategory() != null && responseVo.getProjectCategory().equals(value.getCode())) {
                            responseVo.setProjectCategoryName(value.getInfo());
                            break;
                        }
                    }
                    //承包方式
                    for (EngineeringContractingMode value : EngineeringContractingMode.values()) {
                        if (responseVo.getContractingMode() != null && responseVo.getContractingMode().equals(value.getCode())) {
                            responseVo.setContractingModeName(value.getInfo());
                            break;
                        }
                    }
                    //施工图是否完成
                    for (ConstructionDrawing value : ConstructionDrawing.values()) {
                        if (responseVo.getConstructionDrawing() != null && responseVo.getConstructionDrawing().equals(value.getCode())) {
                            responseVo.setConstructionDrawingName(value.getInfo());
                            break;
                        }
                    }
                    HashMap<String, String> fileMap4 = getFileMap(vo.getDemandId(),SysFileRefEnum.PM_DEMAND_ENGINEERING.getType());
                    if(fileMap4 != null){
                        responseVo.setFileMap(fileMap4);
                    }
                    vo.setPmDemandEngineeringResponseVo(responseVo);
                }
            }

            return vo;
        } else {
            return null;
        }
    }

    /**
     * 查询采购需求列表
     *
     * @param pmReqVo
     * @return 采购需求
     */
    @Override
    public List<PmDemandResVo> selectPmDemandList(PmReqVo pmReqVo) {
        PmDemand pmDemand = new PmDemand();
        pmDemand.setProjectName(pmReqVo.getProjectName());
        pmDemand.setPurchaseServices(pmReqVo.getPurchaseServiceStation());
        pmDemand.setProjectStatus(pmReqVo.getProjectStatus());
        pmDemand.setIsExcess(pmReqVo.getIsExcess());
        Map<String,Object> params = new HashMap<>();
        params.put("beginDate",pmReqVo.getBeginDate());
        params.put("endDate",pmReqVo.getEndDate());
        pmDemand.setParams(params);
        List<PmDemand> pmDemandList = pmDemandMapper.selectPmDemandList(pmDemand);
        List<PmDemandResVo> pmDemandResponseVoList = new ArrayList<>();
        if(pmDemandList != null && !pmDemandList.isEmpty()){
            for(PmDemand pmDemand1 : pmDemandList){
                PmDemandResVo vo = new PmDemandResVo();
                BeanUtils.copyBeanProp(pmDemand1,vo);
                SysDeptResponseVo sysDeptResponseVo = (SysDeptResponseVo) deptService.selectDeptById(vo.getPurchaseDeptId()).get("sysDept");
                if(sysDeptResponseVo != null){
                    vo.setPurchaseDeptName(sysDeptResponseVo.getDeptName());
                }

                String purchaseServicesName = dictDataService.selectDictLabel("purchase_services",vo.getPurchaseServices());
                if(StringUtils.isNotEmpty(purchaseServicesName)){
                    vo.setPurchaseServicesName(purchaseServicesName);
                }

                //项目类型
                for (ProjectTypes value : ProjectTypes.values()) {
                    if (vo.getProjectType() != null && vo.getProjectType().equals(value.getCode())) {
                        vo.setProjectTypeName(value.getInfo());
                        break;
                    }
                }
                //是否为超限额计划
                for (IsExcess value : IsExcess.values()) {
                    if (vo.getIsExcess() != null && vo.getIsExcess().equals(value.getCode())) {
                        vo.setIsExcessName(value.getInfo());
                        break;
                    }
                }

                //采购方式
                for (PlanPurchaseMode value : PlanPurchaseMode.values()) {
                    if (vo.getPurchaseMode() != null && vo.getPurchaseMode().equals(value.getCode())) {
                        vo.setPurchaseModeName(value.getInfo());
                        break;
                    }
                }
                //项目属性
                for (ProjectAttribute value : ProjectAttribute.values()) {
                    if (vo.getProjectAttr() != null && vo.getProjectAttr().equals(value.getCode())) {
                        vo.setProjectAttrName(value.getInfo());
                        break;
                    }
                }
                //预警状态
                for (WarnStatus value : WarnStatus.values()) {
                    if (vo.getWarnStatus() != null && vo.getWarnStatus().equals(value.getCode())) {
                        vo.setWarnStatusName(value.getInfo());
                        break;
                    }
                }

                //项目状态
                for (PmProjectStatus value : PmProjectStatus.values()) {
                    if (vo.getProjectStatus() != null && vo.getProjectStatus().equals(value.getCode())) {
                        vo.setProjectStatusName(value.getInfo());
                        break;
                    }
                }
                pmDemandResponseVoList.add(vo);
            }
        }
        return pmDemandResponseVoList;
    }

    /**
     * 新增采购需求
     *
     * @param pmDemand 采购需求
     * @return 结果
     */
    @Override
    public int insertPmDemand(PmDemand pmDemand) {
        pmDemand.setCreateTime(DateUtils.getNowDate());
        return pmDemandMapper.insertPmDemand(pmDemand);
    }

    /**
     * 修改采购需求
     *
     * @param pmDemand 采购需求
     * @return 结果
     */
    @Override
    public int updatePmDemand(PmDemand pmDemand) {
        pmDemand.setUpdateTime(DateUtils.getNowDate());
        return pmDemandMapper.updatePmDemand(pmDemand);
    }

    /**
     * 需求建档
     *
     * @param pmBookBuildingReqVo 采购需求
     * @return 结果
     */
    @Override
    public int bookBuilding(PmBookBuildingReqVo pmBookBuildingReqVo) {
        PmDemand pmDemand = new PmDemand();
        pmDemand.setDemandId(pmBookBuildingReqVo.getDemandId());
        pmDemand.setUpdateTime(DateUtils.getNowDate());
        return pmDemandMapper.updatePmDemand(pmDemand);
    }

    private HashMap<String, String> getFileMap(Long redId,Integer type){
        HashMap<String, Object> map = new HashMap<>();
        map.put("red_id", redId);
        map.put("type", type);
        List<SysFileRef> fileRefs = sysFileRefService.listByMap(map);
        if (!ObjectUtils.isEmpty(fileRefs)) {
            HashMap<String, String> fileMap = new HashMap<>();
            for (SysFileRef ref : fileRefs) {
                SysFileInfo fileInfo = fileService.getById(ref.getFileId());
                fileMap.put(fileInfo.getFileUrl(), fileInfo.getFileName());
            }
            return fileMap;
        } else {
            return null;
        }

    }


    /**
     * 查看流程图
     *
     * @param demandId 采购需求主键
     * @return 采购需求
     */
    @Override
    public PmDemandResVo viewFlowChart(Long demandId) {
        PmDemandResVo pmDemandResVo = new PmDemandResVo();
        return pmDemandResVo;
    }

    /**
     * 提交采购需求
     *
     * @param pmReqVo
     * @return 结果
     */
    @Override
    public int commit(PmReqVo pmReqVo) {
        PmDemand pmDemand = new PmDemand();
        pmDemand.setDemandId(pmReqVo.getDemandId());
        pmDemand.setProjectStatus(Long.parseLong(PmProjectStatus.DEMAND_WAIT_AUDIT.getCode()));
        pmDemand.setUpdateTime(DateUtils.getNowDate());
        return pmDemandMapper.updatePmDemand(pmDemand);
    }

    /**
     * 审核采购需求通过
     *
     * @param pmReqVo
     * @return 结果
     */
    @Override
    public int reviewTo(PmReqVo pmReqVo) {
        PmDemand pmDemand = new PmDemand();
        pmDemand.setDemandId(pmReqVo.getDemandId());
        pmDemand.setProjectStatus(Long.parseLong(PmProjectStatus.TASK_WAIT_RELEASE.getCode()));
        List<SysFileRef> sysFileRefs = pmReqVo.getSysFileRefs();
        if (!ObjectUtils.isEmpty(sysFileRefs)) {
            for (SysFileRef ref : sysFileRefs) {
                ref.setRedId(pmReqVo.getDemandId());
                ref.setType(SysFileRefEnum.PM_DEMAND_EXAMINE.getType());
                ref.setCreated(pmReqVo.getUpdateBy());
                ref.setCreateTime(new Date());
                ref.setUpdated(pmReqVo.getUpdateBy());
                ref.setUpdateTime(new Date());
                sysFileRefService.save(ref);
            }
        }
        pmDemand.setUpdateTime(DateUtils.getNowDate());
        pmDemand.setAuditTime(DateUtils.getNowDate());
        return pmDemandMapper.updatePmDemand(pmDemand);
    }

    /**
     * 审核采购需求退回
     *
     * @param pmReqVo
     * @return 结果
     */
    @Override
    public int reviewReturn(PmReqVo pmReqVo) {
        PmDemand pmDemand = new PmDemand();
        pmDemand.setDemandId(pmReqVo.getDemandId());
        pmDemand.setProjectStatus(Long.parseLong(PmProjectStatus.DEMAND_WAIT_COMMIT.getCode()));
        pmDemand.setAdjustReason(pmReqVo.getRefuseReason());
        pmDemand.setUpdateTime(DateUtils.getNowDate());
        pmDemand.setAuditTime(DateUtils.getNowDate());
        return pmDemandMapper.updatePmDemand(pmDemand);
    }
}
